<?php

namespace Drupal\component_builder_toolbar;

use Drupal\Component\Serialization\Json;
use Drupal\component_builder\ComponentBuilderHelperInterface;
use Drupal\component_builder\Entity\ComponentWrapperInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;

class ToolbarManager implements ToolbarManagerInterface {

  use LoggerChannelTrait;

  /**
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * @var \Drupal\component_builder\ComponentBuilderHelperInterface
   */
  protected $componentBuilderHelper;

  /**
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   * @param \Drupal\component_builder\ComponentBuilderHelperInterface $component_builder_helper
   */
  public function __construct(RouteMatchInterface $route_match, AccountProxyInterface $current_user, ComponentBuilderHelperInterface $component_builder_helper) {
    $this->routeMatch = $route_match;
    $this->currentUser = $current_user;
    $this->componentBuilderHelper = $component_builder_helper;
  }


  /**
   * {@inheritdoc}
   */
  public function accessToolbar($return_info = FALSE, $route_suffix = 'component_builder_toolbar') {
    $route_name = $this->routeMatch->getRouteName();

    if ($referenced_entities = $this->componentBuilderHelper->getFieldsReferencedComponent()) {
      $entity_canonical_route_names = array_map(function($entity_type_id) use ($route_suffix) {
        return "entity.{$entity_type_id}.{$route_suffix}";
      }, array_keys($referenced_entities));

      if (in_array($route_name, $entity_canonical_route_names)) {
        $entity_type_id = explode('.', $route_name)[1];
        /** @var \Drupal\Core\Entity\EntityInterface $entity */
        $entity = $this->routeMatch->getParameter($entity_type_id);

        if (isset($referenced_entities[$entity->getEntityTypeId()][$entity->bundle()]) && $entity->access('update')) {
          if ($return_info) {
            return $referenced_entities[$entity->getEntityTypeId()][$entity->bundle()];
          }

          return TRUE;
        }
      }
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function moveWrapper(EntityInterface $entity, ComponentWrapperInterface $component_wrapper, $field_name, $from_region, $from, $to_region, $to) {
    if ($entity instanceof FieldableEntityInterface && $entity->hasField($field_name)) {
      $referenced_wrapper_values = $entity->get($field_name)->getValue();
      $wrapper_ids = array_column($referenced_wrapper_values, 'target_id');
      if ($to_region == 'content') {
        $tmp = array_splice($wrapper_ids, $from, 1);
        array_splice($wrapper_ids, $to, 0, $tmp);
        try {
          $component_wrapper->get('display_in_region')->setValue(FALSE);
          $component_wrapper->save();

          $entity->get($field_name)->setValue($wrapper_ids);
          $entity->save();
          return TRUE;
        }
        catch (\Exception $e) {
          $this->getLogger('component_builder_toolbar')->error($e->getMessage());
          return FALSE;
        }
      }
      else {
        try {
          $component_wrapper->get('display_in_region')->setValue(TRUE);
          $component_wrapper->get('field_region')->setValue($to_region);
          $component_wrapper->get('field_weight_in_region')->setValue($to);
          $component_wrapper->save();
          return TRUE;
        }
        catch (\Exception $e) {
          $this->getLogger('component_builder_toolbar')->error($e->getMessage());
          return FALSE;
        }
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function buildComponentWrapperMarkup(EntityInterface $entity, ComponentWrapperInterface $component_wrapper, Url $move_wrapper_url = NULL, $delta = 0) {
    $region_weight = $delta;
    $display_in_region = $component_wrapper->get('display_in_region')->value;
    if ($display_in_region) {
      $region_weight = $component_wrapper->get('field_weight_in_region')->value;
    }

    $entity_type_id = $entity->getEntityTypeId();
    $move_wrapper_url_params = $move_wrapper_url->getRouteParameters();
    $entity_detail = \Drupal::routeMatch()->getParameter($entity_type_id);
    if ($entity_detail instanceof EntityInterface) {
      $move_wrapper_url_params[$entity_type_id] = $entity_detail->id();
    }
    $edit_link = Url::fromRoute(
      "entity.{$entity->getEntityTypeId()}.component_builder_toolbar_edit_component",
      $move_wrapper_url_params
    );

    $view_builder = \Drupal::entityTypeManager()->getViewBuilder($component_wrapper->getEntityTypeId());
    $wrapper_preview = $view_builder->view($component_wrapper, 'full', $component_wrapper->language()->getId());
    return [
      '#theme' => 'component_builder_toolbar_wrapper',
      '#wrapper_id' => $component_wrapper->id(),
      '#move_wrapper_url' => $move_wrapper_url->toString(),
      '#title' => $component_wrapper->label(),
      '#field_delta' => $delta,
      '#region_weight' => $region_weight,
      '#edit_link' => [
        '#type' => 'link',
        '#title' => t('Edit'),
        '#url' => $edit_link,
        '#attributes' => [
          'class' => ['use-ajax', 'component-builder-toolbar__wrapper-edit-link'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => Json::encode([
            'width' => '1200',
          ]),
        ],
      ],
      '#wrapper_preview' => $wrapper_preview,
      '#weight' => $region_weight,
      '#cache' => [
        'tags' => [
          'component_builder_toolbar:component_wrapper:' . $entity->id() . ':' . $component_wrapper->id(),
        ]
      ]
    ];
  }

}
